-----------------------------------------------------------------------
-- Locale
--

local PREFIX = "cg: "
local CG_USAGE = "Whisper me '!cg <anything>' to search."
local NO_DATA = "No results found for %q."
local acceptedCommands = {
	cg = true,
	craft = true,
	enchant = true,
	gem = true,
}

-----------------------------------------------------------------------
-- Tooltip scanner, mostly stolen from Gratuity.
--

local scanItem
do
	local scanner = CreateFrame("GameTooltip")
	scanner:SetOwner(WorldFrame, "ANCHOR_NONE")
	local lcache, rcache = {}, {}
	for i = 1, 30 do
		lcache[i], rcache[i] = scanner:CreateFontString(), scanner:CreateFontString()
		lcache[i]:SetFontObject(GameFontNormal); rcache[i]:SetFontObject(GameFontNormal)
		scanner:AddFontStrings(lcache[i], rcache[i])
	end
	local function erase()
		for i = 1, scanner:NumLines() do rcache[i]:SetText() end
		scanner:ClearLines()
	end
	local scannerCache = {}
	function scanItem(itemLink)
		erase()
		scanner:SetHyperlink(itemLink)
		for k in pairs(scannerCache) do scannerCache[k] = nil end
		for i = 1, scanner:NumLines() do
			local left, right = lcache[i]:GetText(), rcache[i]:GetText()
			if left then table.insert(scannerCache, left) end
			if right then table.insert(scannerCache, right) end
		end
		return table.concat(scannerCache)
	end
end

-----------------------------------------------------------------------
-- CraftyGnome
--

local db = nil
local cache = {}
local addon = CreateFrame("Frame")

function addon:CRAFT_SHOW()
	for i = 1, GetNumCrafts() do
		local skillName, _, skillType = GetCraftInfo(i)
		if skillType ~= "header" and skillName then
			local link = GetCraftRecipeLink(i)
			if not db[link] then
				db[link] = true
			end
		end
	end
end

function addon:TRADE_SKILL_SHOW()
	local name = GetTradeSkillLine()
	for i = 1, GetNumTradeSkills() do
		local skillName, skillType = GetTradeSkillInfo(i)
		if skillType ~= "header" and skillName then
			local link = GetTradeSkillRecipeLink(i)
			if not db[link] then
				db[link] = true
			end
		end
	end
end

do
	local ctl = ChatThrottleLib
	local reply = ctl and
		function(msg, player) ctl:SendChatMessage("BULK", "CraftyGnome", PREFIX .. msg, "WHISPER", nil, player) end or
		function(msg, player) SendChatMessage(PREFIX .. msg, "WHISPER", nil, player) end

	local function updateCache()
		for item in pairs(db) do
			if not cache[item] then
				cache[item] = scanItem(item):lower()
			end
		end
	end

	local function processChat(msg, player, isWhisper)
		local cmd = select(3, msg:find("^!(%S+)"))
		if not cmd or not acceptedCommands[cmd] then return end
		local query = select(3, msg:find("^!%S+%s(.*)"))
		if query then
			local name = GetItemInfo(query)
			if name then query = name:lower()
			else query = query:lower() end
			updateCache()
			local nothing = true
			for item, details in pairs(cache) do
				if details:find(query) then
					nothing = nil
					reply(item, player)
				end
			end
			if isWhisper and nothing then
				reply(NO_DATA:format(query), player)
			end
		elseif isWhisper then
			reply(CG_USAGE, player)
		end
	end
	function addon:CHAT_MSG_GUILD(msg, player)
		processChat(msg, player)
	end
	function addon:CHAT_MSG_WHISPER(msg, player)
		processChat(msg, player, true)
	end
end

function addon:PLAYER_ENTERING_WORLD()
	if not CraftyGnomeDB then CraftyGnomeDB = {} end
	local realm = GetRealmName()
	if not CraftyGnomeDB[realm] then CraftyGnomeDB[realm] = {} end
	local faction = UnitFactionGroup("player")
	if not CraftyGnomeDB[realm][faction] then CraftyGnomeDB[realm][faction] = {} end
	db = CraftyGnomeDB[realm][faction]
	addon:UnregisterEvent("PLAYER_ENTERING_WORLD")
end

local orig = ChatFrame_MessageEventHandler
local chatPrefixTest = "^" .. PREFIX
ChatFrame_MessageEventHandler = function(event, ...)
	if event == "CHAT_MSG_WHISPER_INFORM" and type(arg1) == "string" and arg1:find(chatPrefixTest) then
		return
	end
	return orig(event, ...)
end

addon:SetScript("OnEvent", function(self, event, ...)
	self[event](self, ...)
end)
addon:RegisterEvent("PLAYER_ENTERING_WORLD")
addon:RegisterEvent("CHAT_MSG_GUILD")
addon:RegisterEvent("CHAT_MSG_WHISPER")
addon:RegisterEvent("CRAFT_SHOW")
addon:RegisterEvent("TRADE_SKILL_SHOW")

